@@ -19,15 +19,16 @@ import os
 import os.path
 
 PACKAGE    = "sabayon"
-VERSION    = "2.29.2"
-LOCALEDIR = "/usr/local/share/locale"
+VERSION    = "2.29.5"
+LOCALEDIR  = "/usr/local/share/locale"
 DATADIR    = "/usr/local/share"
 SYSCONFDIR = "/usr/local/etc"
 LIBEXECDIR = "/usr/local/libexec"
 SBINDIR    = "/usr/local/sbin"
 
-GLADEDIR    = os.path.join (DATADIR, "sabayon", "glade")
-PROFILESDIR = os.path.join (SYSCONFDIR, "desktop-profiles")
+BUILDERDIR  = os.path.join (DATADIR,    "sabayon", "ui")
+CONFIGDIR   = os.path.join (SYSCONFDIR, "sabayon")
+PROFILESDIR = os.path.join (CONFIGDIR,  "profiles")
 
 MONITOR_TOOL_ARGV = [ os.path.join (LIBEXECDIR, "sabayon-monitor") ]
 SESSION_TOOL_ARGV = [ os.path.join (LIBEXECDIR, "sabayon-session") ]
@@ -157,6 +158,6 @@ GCONF_KEYS_TO_IGNORE = [
     "/desktop/gnome/peripherals/keyboard/kbd.sysbackup/*",
 ]
 
-LOG_CONFIG_FILENAME = "sabayon-debug-log.conf"
+LOG_CONFIG_FILENAME = os.path.join (CONFIGDIR, "sabayon-debug-log.conf")
 
 BUG_TRACKING_SYSTEM = "http://bugzilla.gnome.org"
