@@ -92,7 +92,7 @@ class ProfileStorage:
     fact that there is a temporary directory are both
     implementation details and not exposed in the API.
 
-    Profile files are stored in /etc/desktop-profiles.
+    Profile files are stored in /etc/sabayon/profiles.
 
     Each file or directory in the profile has metadata
     associated with it - the "source" of the file/directory
@@ -104,7 +104,7 @@ class ProfileStorage:
         """Create a ProfileStorage.
 
         @name: the name of the profile - translates to
-        /etc/desktop-profiles/$(name).zip.
+        /etc/sabayon/profiles/$(name).zip.
         """
         self.name = name
         self.readonly = 0
@@ -136,7 +136,7 @@ class ProfileStorage:
 
     def __del__ (self):
         if self.temp_path:
-            shutil.rmtree (self.temp_path)
+            shutil.rmtree (self.temp_path, True)
         self.temp_path = None
 
         if self.metadata:
@@ -151,6 +151,7 @@ class ProfileStorage:
         metadata = libxml2.newDoc ("1.0")
         root = metadata.newChild (None, "metadata", None)
         root.newChild (None, "directories", None)
+        root.newChild (None, "links", None)
         root.newChild (None, "files", None)
         return metadata
 
@@ -257,6 +258,24 @@ class ProfileStorage:
 
         self.__update_file_or_dir_node (directory_node, source, attributes, metadata)
 
+    def __update_link_node (self, path, source, attributes, metadata = None):
+        if metadata is None:
+            metadata = self.metadata
+        assert metadata
+
+        files_node = metadata.xpathEval ("/metadata/links")[0]
+
+        # Ensure the directory node exists
+        nodes = files_node.xpathEval ("link[@path='%s']" % path)
+        if len (nodes):
+            link_node = nodes[0]
+        else:
+            link_node = files_node.newChild (None, "link", None)
+            link_node.setProp ("path", path)
+            link_node.setProp ("dest", os.readlink(path))
+
+        self.__update_file_or_dir_node (link_node, source, attributes, metadata)
+
     def __unpack (self):
         self.__read_metadata ()
 
@@ -284,7 +303,7 @@ class ProfileStorage:
                     os.makedirs (dest_dir)
 
                 # It sucks that we lose file permissions, mtime etc. with ZIP
-                file (dest_path, "w").write (zip.read (f))
+                zip.extract(f, dir)
 
         def unzip_foreach (path, is_directory, data):
             (zip, temp_path) = data
@@ -302,7 +321,7 @@ class ProfileStorage:
                     os.makedirs (dest_dir)
 
                 # It sucks that we lose file permissions, mtime etc. with ZIP
-                file (abs_path, "w").write (zip.read (path))
+                zip.extract(path, temp_path)
 
         self.__foreach_all (unzip_foreach, (self.zip, self.temp_path))
 
@@ -359,7 +378,10 @@ class ProfileStorage:
         dst_path = os.path.join (self.temp_path, path)
 
         if not os.path.exists (src_path):
-            raise ProfileStorageException (_("Cannot add non-existent file '%s'") % src_path)
+            # change the raise to a dprint, since some files seem to disappear.
+            # raise ProfileStorageException (_("Cannot add non-existent file '%s'") % src_path)
+            dprint (_("Cannot add non-existent file '%s'"), src_path)
+            return
 
         # Remove old version
         node = self.__get_node (path)
@@ -371,7 +393,9 @@ class ProfileStorage:
         if os.path.isdir (src_path):
             self.__update_directory_node (path, source, attributes)
             copy_tree (self.temp_path, src_dir, path)
-        else:
+        elif os.path.islink (src_path):
+            self.__update_link_node (path, source, attributes)
+        elif os.path.isfile (src_path) and not os.path.islink (src_path):
             self.__update_file_node (path, source, attributes)
             dirname = os.path.dirname (dst_path)
             if not os.path.exists (dirname):
@@ -389,6 +413,15 @@ class ProfileStorage:
             return dir_nodes[0]
         return None
 
+    def __get_link_node (self, path, metadata = None):
+        if metadata is None:
+            metadata = self.metadata
+        assert metadata
+        link_nodes = metadata.xpathEval ("/metadata/links/link[@path='%s']" % path)
+        if len (link_nodes) > 0:
+            return link_nodes[0]
+        return None
+
     def __get_file_node (self, path, metadata = None):
         if metadata is None:
             metadata = self.metadata
@@ -402,6 +435,9 @@ class ProfileStorage:
         self.__read_metadata ()
 
         node = self.__get_dir_node (path, metadata)
+        if node:
+            return node
+        node = self.__get_link_node (path, metadata)
         if node:
             return node
         node = self.__get_file_node (path, metadata)
@@ -436,6 +472,9 @@ class ProfileStorage:
         node = self.__get_dir_node (path)
         if node:
             return "directory"
+        node = self.__get_link_node (path)
+        if node:
+            return "link"
         node = self.__get_file_node (path)
         if node:
             return "file"
@@ -481,8 +520,16 @@ class ProfileStorage:
                     raise err
 
             if got_stat:
-                os.unlink (dest) # FIXME: this could fail, but that would be because the parent
-                                 # directory is not writable.  Then we have bigger problems, anyway.
+                if os.path.isdir (dest):
+                    try:
+                        os.rmdir (dest)
+                    except OSError, err:
+                        raise ProfileStorageException (_("Couldn't rmdir '%s'") % dest)
+                else:
+                    try:
+                        os.unlink (dest)
+                    except OSError, err:
+                        raise ProfileStorageException (_("Couldn't unlink file '%s'") % dest)
 
             # FIXME: we lose the "original" permissions, mtime, etc. with ZIP files.
             shutil.copy2 (src, dest)
@@ -498,6 +545,10 @@ class ProfileStorage:
 
         if item_type == "directory":
             copy_tree (dst_dir, self.temp_path, path, None, overwrite)
+        elif item_type == "link":
+            link_node = self.__get_link_node (path)
+            dest = link_node.prop("dest")
+            os.symlink(dest, path)
         else:
             dst_path = os.path.join (dst_dir, path)
             if overwrite or not os.path.exists (dst_path):
@@ -554,15 +605,18 @@ class ProfileStorage:
         """
         self.__read_metadata ()
 
-        for node in self.metadata.xpathEval ("/metadata/files/file"):
+        for node in self.metadata.xpathEval ("/metadata/directories/directory"):
             self.__foreach_node (node, callback, user_data, source)
 
-        for node in self.metadata.xpathEval ("/metadata/directories/directory"):
+        for node in self.metadata.xpathEval ("/metadata/links/link"):
+            self.__foreach_node (node, callback, user_data, source)
+
+        for node in self.metadata.xpathEval ("/metadata/files/file"):
             self.__foreach_node (node, callback, user_data, source)
 
     def save (self):
         """Save the contents of the profile to
-        /etc/desktop-profiles/$(name).zip.
+        /etc/sabayon/profiles/$(name).zip.
         """
         if self.readonly:
             raise ProfileStorageException (_("Profile is read-only %s") %
@@ -600,20 +654,23 @@ class ProfileStorage:
                     if os.path.isdir (path):
                         # We need to stop if we are recursing inside a ignored 
                         # directory.
-                        if util.should_ignore_dir (homedir, 
-                                DIRECTORIES_TO_IGNORE, os.path.join(homedir, name, f)):
+                        if util.should_ignore_dir (homedir,
+                                                   DIRECTORIES_TO_IGNORE_PROFILE,
+                                                   os.path.join (homedir, name, f)):
                             dprint ("Not going inside %s as it is an ignored directory.", path)
                         else:
                             zip_directory (save_zip,
                                         path,
                                         os.path.join (name, f))
-                    elif os.path.isfile (path):
+                    elif os.path.isfile (path) and not os.path.islink (path):
                         # Avoid putting in a duplicate file entry
                         # See bug #476761
                         if os.path.join (name, f) in zip_filelist:
                             dprint ("Not adding %s to zipfile since it is already in the file", os.path.join (name, f))
-                        elif util.should_ignore_file (homedir, 
-                                DIRECTORIES_TO_IGNORE, FILES_TO_IGNORE, os.path.join(homedir, name, f)):
+                        elif util.should_ignore_file (homedir,
+                                                      DIRECTORIES_TO_IGNORE_PROFILE,
+                                                      FILES_TO_IGNORE,
+                                                      os.path.join (homedir, name, f)):
                             dprint ("Not adding %s to zipfile since it is a ignored file", os.path.join (name, f))
                         else:
                             zip_filelist.append(os.path.join(name, f))
@@ -635,13 +692,12 @@ class ProfileStorage:
         except:
             if backup:
                 failsafe_rename (backup, self.path)
-            raise
 
         if backup:
             os.remove (backup)
 
         if self.temp_path:
-            shutil.rmtree (self.temp_path)
+            shutil.rmtree (self.temp_path, True)
             self.temp_path = None
 
         self.needs_saving = False
@@ -779,7 +835,7 @@ def run_unit_tests ():
     assert file (os.path.join (temp_dir2, "foobar/foo.txt")).read () == "new test file 6"
 
     # Remove temporary extraction dir
-    shutil.rmtree (temp_dir2)
+    shutil.rmtree (temp_dir2, True)
 
     ########## Create second revision
 
@@ -803,7 +859,7 @@ def run_unit_tests ():
     profile.add ("blaas", temp_dir, "Waterford", { "nice" : True, "with-butter" : "but of course" })
 
     # Remove temporary dir
-    shutil.rmtree (temp_dir)
+    shutil.rmtree (temp_dir, True)
 
     # Save the profile (second revision)
     os.remove (profile_path)
@@ -871,7 +927,7 @@ def run_unit_tests ():
     assert file (os.path.join (temp_dir2, "blaas/are/nice/foo.txt")).read () == "blaas are nice"
 
     # Remove temporary extraction dir
-    shutil.rmtree (temp_dir2)
+    shutil.rmtree (temp_dir2, True)
 
     ########## Create third revision
 
@@ -894,7 +950,7 @@ def run_unit_tests ():
     assert os.path.exists (profile_path)
 
     # Remove temp dir again
-    shutil.rmtree (temp_dir)
+    shutil.rmtree (temp_dir, True)
 
     ########## Verify the third revision
 
@@ -955,6 +1011,6 @@ def run_unit_tests ():
     assert file (os.path.join (temp_dir, "foobar/foo.txt")).read () == "new test file 2005"
 
     # Remove temporary extraction dir
-    shutil.rmtree (temp_dir)
+    shutil.rmtree (temp_dir, True)
 
     os.remove (profile_path)
